<?php
function bundle_clone_admin_display($form, &$form_state, $entity_type) {
  $entity_bundles = field_info_bundles($entity_type);
  $form['entity_type'] = array(
    '#type' => 'fieldset',
    '#title' => $entity_type,
    '#tree' => TRUE,
    '#prefix' => l('Back to entity select', 'admin/structure/bundle_clone'),
  );
  
  $form['entity_type']['type'] = array(
    '#type' => 'value',
    '#value' => $entity_type,
  );

  $form['entity_type']['bundles'] = array(
    '#type' => 'value',
    '#value' => $entity_bundles,
  );

  $form['entity_type'] += bundle_clone_entity_items($entity_bundles);

  $info = entity_get_info($entity_type);
  $options = array();
  foreach ($info['view modes'] as $mode => $definition) {
    $options[$mode] = $definition['label'];
  }
  $form['entity_type']['view_mode'] = array(
    '#type' => 'select',
    '#title' => t('View mode'),
    '#options' => $options,
  );

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Update display'),
  );
    
  return $form;
}

function bundle_clone_admin_display_validate(&$form, &$form_state) {
  bundle_clone_validate_selection($form, $form_state);
}

function bundle_clone_admin_display_submit(&$form, &$form_state) {
  $source = $form_state['values']['entity_type']['source'];
  $target = $form_state['values']['entity_type']['target'];
  $entity_type = $form_state['values']['entity_type']['type'];
  $bundles = $form_state['values']['entity_type']['bundles'];
  $view_mode = $form_state['values']['entity_type']['view_mode'];
  $holders = array(
    '!source' => l($bundles[$source]['label'], $bundles[$source]['admin']['real path']),
    '!target' => l($bundles[$target]['label'], $bundles[$target]['admin']['real path']),
    '@view_mode' => $form['entity_type']['view_mode']['#options'][$view_mode],
  );
  drupal_set_message(t('Updating display @view_mode !source into !target', $holders));
  bundle_clone_update_display($entity_type, $source, $target, $view_mode);
}

function bundle_clone_update_display($entity_type, $source, $target, $view_mode) {
  $instances = field_info_instances($entity_type, $source);
  $target_instances = field_info_instances($entity_type, $target);
  foreach ($target_instances as $instance => $config) {
    if (!isset($instances[$instance])) {
      drupal_set_message(t('Skipping @instance', array('@instance' => $instance)));
      continue;
    }
    drupal_set_message(t('Updating @instance', array('@instance' => $instance)));
    if (!isset($config['display'][$view_mode])) {
      $view_mode = 'default';
    }
    $config['display'][$view_mode] = 
      $instances[$instance]['display'][$view_mode] + $config['display'][$view_mode];
    field_update_instance($config);
  }
}

